/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgram
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive JFrame v1");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleState cs = new CircleState();
      final CirclePanel cp = new CirclePanel(cs);
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.requestFocusInWindow();
      jf.setVisible(true);

      jf.addKeyListener(new CircleListener(cs, cp));
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleState cs          |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
GUI object that receives        |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM.                                        |                         |
                                                +-------------------------+
                                                 An object that implements
                                                 the KeyListener interface.
    KeyEvent object
  +-----------------+
  |  information    |
  |  from the OS    |      CircleState object         CirclePanel object
  |  about the      |      +-----------------+       +------------------+
  |  keyboard event |      |  int xNew       |       | CircleState cs   |
  +-----------------+      |  int yNew       |       |                  |
                           |  int xOld       |       |                  |
                           |  int yOld       |       | paintComponent() |
                           +-----------------+       +------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
