/*

*/

import java.awt.event.*;

/**
   This class has the responsibility of responding to
   key events by updating the state (data) of our
   graphics program and then notifying the graphics
   part of program that it needs to redraw itself (using
   the new state information).

   The methods from the KeyListener interface will be
   called by the Event Dispatch Thread (the "GUI thread")
   whenever the program's user causes the appropriate event.

   In this version of the program, this class acts as the
   data object and it holds the state of the program.
*/
public class CircleListener implements KeyListener
{
   public int xNew = 300; // where the circle needs to be
   public int yNew = 300;
   public int xOld = 300; // where the circle is now
   public int yOld = 300;
   private CirclePanel cp;

   public void setPanel(CirclePanel cp)
   {
      this.cp = cp;
   }


   // implement the KeyListener interface (three methods)
   @Override public void keyPressed(KeyEvent e)
   {
      final int keyCode = e.getKeyCode();
      if (keyCode == KeyEvent.VK_UP)
      {
         yNew -= 5;      // update the state
         cp.repaint();   // notify the view
      }
      else if (keyCode == KeyEvent.VK_DOWN)
      {
         yNew += 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_LEFT)
      {
         xNew -= 5;
         cp.repaint();
      }
      else if (keyCode == KeyEvent.VK_RIGHT)
      {
         xNew += 5;
         cp.repaint();
      }
   }
   @Override public void keyReleased(KeyEvent e){ }
   @Override public void keyTyped(KeyEvent e){ }
}
