/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.
*/
public class CircleProgram
{
   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive JFrame v2b");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CircleListener cs = new CircleListener();
      final CirclePanel cp = new CirclePanel(cs);
      cs.setPanel(cp);
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      jf.addKeyListener(cs);
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|                      |    List<KeyListener>     CircleListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | int xNew                |
|                      |        +------+        | int yNew                |
|  addKeyListener()    |        |  ----|------->| int xOld                |
+----------------------+        +------+        | int yOld                |
GUI object that receives        |      |        | CirclePanel cp          |
KeyEvent objects from           +------+        |                         |
the JVM.                                        | keyPressed(KeyEvent e)  |
                                                | keyReleased(KeyEvent e) |
                                                | keyTyped(KeyEvent e)    |
    KeyEvent object                             |                         |
  +-----------------+                           +-------------------------+
  |  information    |                            An object that implements
  |  from the OS    |                            the KeyListener interface
  |  about the      |                            and stores program state.
  |  keyboard event |
  +-----------------+                                 CirclePanel object
                                                     +-------------------+
                                                     | CircleListener cs |
                                                     |                   |
                                                     |                   |
                                                     | paintComponent()  |
                                                     +-------------------+
                                                     GUI object that needs
                                                     to be repainted after
                                                     an event occurs.
*/
