/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class builds the GUI, instantiates
   all the program objects, and makes sure
   that every object has a reference to any
   other object that it must communicate with.

   This class instantiates the JFrame that
   is the source of the key events.

   In this version, this program object acts as
   the data object and it holds the state of
   the program.
*/
public class CircleProgram
{
   public int xNew = 300; // where the circle needs to be
   public int yNew = 300;
   public int xOld = 300; // where the circle is now
   public int yOld = 300;

   /**
      This constructor instantiates the GUI.
   */
   public CircleProgram()
   {
      final JFrame jf = new JFrame("Interactive JFrame v2c");
      jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      final CirclePanel cp = new CirclePanel(this);
      jf.getContentPane().add(cp);
      jf.pack();
      jf.setLocationRelativeTo(null);
      jf.setVisible(true);

      jf.addKeyListener(new CircleListener(this, cp));
   }//constructor

   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      // We need to call the program's constructor in the
      // Java GUI Event Dispatch Thread, otherwise we get a
      // race condition between the constructor (running in
      // the main() thread) and the first ComponentEvent
      // (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         () -> new CircleProgram()
      );
   }//main()
}

/*
Here is a picture showing some relevant objects in the JVM's heap.

    JFrame object
+----------------------+
|  int xNew            |
|  int yNew            |
|  int xOld            |
|  int yOld            |
|                      |       KeyListener         CircleListener object
|  List<KeyListener>---|------->+------+        +-------------------------+
|                      |        |      |        | CircleProgram cProg     |
|                      |        +------+        | CirclePanel cp          |
|  addKeyListener()    |        |  ----|------->|                         |
+----------------------+        +------+        | keyPressed(KeyEvent e)  |
an object that receives         |      |        | keyReleased(KeyEvent e) |
KeyEvent objects from           +------+        | keyTyped(KeyEvent e)    |
the JVM                                         |                         |
                                                +-------------------------+
                                                 an object that implements
    KeyEvent object                              the KeyListener interface
  +-----------------+
  |  information    |
  |  from the OS    |                                 CirclePanel object
  |  about the      |                                +------------------+
  |  keyboard event |                                | CircleProgram cp |
  +-----------------+                                |                  |
                                                     |                  |
                                                     | paintComponent() |
                                                     +------------------+
*/
